      program flexpart
********************************************************************************
*                                                                              *
*     This is the Lagrangian Particle Dispersion Model FLEXPART.               *
*     The main program manages the reading of model run specifications, etc.   *
*     All actual computing is done within subroutine timemanager.              *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     18 May 1996                                                              *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* error                .true., if error ocurred in subprogram, else .false.    *
*                                                                              *
* Constants:                                                                   *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer idummy,i
      logical error,oronew

      data idummy/-320/


C Read the pathnames where input/output files are stored
********************************************************

      call readpaths(error)
      if (error) goto 999


C Read the user specifications for the current model run
********************************************************

      call readcommand(error)
      if (error) goto 999
       

C Read the age classes to be used
*********************************

      call readageclasses(error)
      if (error) goto 999
       

C Read, which wind fields are available within the modelling period
*******************************************************************

      call readavailable(error)
      if(error) goto 999


C Read the model grid specifications of the current model run
*************************************************************

      call gridcheck(oronew,error)
      if (error) goto 999
      call gridcheck_nests(error)
      if (error) goto 999


C Read the output grid specifications of the current model run
**************************************************************

      call readoutgrid(error)
      if (error) goto 999


C Read the receptor points for which concentrations are to be calculated
************************************************************************

      call readreceptors(error)
      if (error) goto 999


C Read the chemical species table
*********************************

      call readspecies(error)
      if (error) goto 999


C Read the orography used by the ECMWF model
********************************************

      call readoro(oronew,error)
      if (error) goto 999


C Read the landuse inventory
****************************

      call readlanduse(error)
      if (error) goto 999


C Assign fractional cover of landuse classes to each ECMWF grid point
*********************************************************************

      call assignland()


C Read the coordinates of the release locations for the current model run
*************************************************************************

      call readreleases(error)
      if (error) goto 999


C Read and compute surface resistances to gaseous dry deposition
****************************************************************

      call readdepo(error)
      if (error) goto 999


C Conversion of the startpoints from geografical to grid coordinates
********************************************************************

      call coordtrafo(error)
      if (error) goto 999


C For continuation of previous run, read in particle positions
**************************************************************

      if (ipin.eq.1) then
        call readpartpositions(error)
        if (error) goto 999
      else
        numpart=0
      endif

      write(*,*) 'numpart=',numpart

C Fix the release times and release positions of all particles
**************************************************************

      call fixparticles(error)
      if (error) goto 999

 
C Open the FLEXPART output files
********************************

      if ((iout.eq.1).or.(iout.eq.3))      ! concentration
     +call openoutput(unitoutgrid,unitoutrecept,'grid_conc',
     +'receptor_conc',error)
      if ((iout.eq.2).or.(iout.eq.3))      ! mixing ratio 
     +call openoutput(unitoutgridppt,unitoutreceptppt,'grid_pptv',
     +'receptor_pptv',error)
      if ((ipout.eq.1).or.(ipout.eq.2))    ! particle positions
     +call openoutput(unitpartout,-999,'partposit',
     +'dummyfilename',error)
      if (iflux.eq.1)                      ! fluxes
     +call openoutput(unitflux,-999,'grid_flux',
     +'dummyfilename',error)
      if(error) goto 999


C Generate a large number of random numbers
*******************************************

      do 10 i=1,maxrand-1,2
10      call gasdev1(idummy,rannumb(i),rannumb(i+1))
      call gasdev1(idummy,rannumb(maxrand),rannumb(maxrand-1))


C Calculate particle trajectories
*********************************
      
      call timemanager()

      close(unitoutgrid)
      close(unitoutgridppt)


      write(*,*) 'CONGRATULATIONS: YOU HAVE SUCCESSFULLY COMPLETED A FLE
     +XPART MODEL RUN!'

      goto 1000

999   write(*,*) 'FLEXPART MODEL ERROR: EXECUTION HAD TO BE TERMINATED'

1000  continue

      end
