      subroutine calcpar(n,tth,qvh,uuh,vvh)
C                        i  i   i   i   i
********************************************************************************
*                                                                              *
*     Computation of several boundary layer parameters needed for the          *
*     dispersion calculation and calculation of dry deposition velocities.     *
*     All parameters are calculated over the entire grid.                      *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     21 May 1995                                                              *
*                                                                              *
* ------------------------------------------------------------------           *
*     Petra Seibert, Feb 2000:                                                 *
*     convection scheme:                                                       *
*     new variables in call to richardson                                      *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* n                  temporal index for meteorological fields (1 to 3)         *
*                                                                              *
* Constants:                                                                   *
*                                                                              *
*                                                                              *
* Functions:                                                                   *
* scalev             computation of ustar                                      *
* obukhov            computatio of Obukhov length                              *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      integer n,ix,jy,i
      real ttlev(nuvzmax),qvlev(nuvzmax),obukhov,scalev,ol,hmixplus
      real ulev(nuvzmax),vlev(nuvzmax),ew,rh,vd(maxspec),subsceff,hmix1
      real tth(0:nxmax-1,0:nymax-1,nuvzmax)
      real qvh(0:nxmax-1,0:nymax-1,nuvzmax)
      real uuh(0:nxmax-1,0:nymax-1,nuvzmax)
      real vvh(0:nxmax-1,0:nymax-1,nuvzmax)


C Loop over entire grid
***********************

      do 10 jy=0,ny-1
        do 10 ix=0,nx-1

C 1) Calculation of friction velocity
*************************************

          ustar(ix,jy,1,n)=scalev(ps(ix,jy,1,n),tt2(ix,jy,1,n),
     +    td2(ix,jy,1,n),surfstr(ix,jy,1,n))
          if (ustar(ix,jy,1,n).le.1.e-8) ustar(ix,jy,1,n)=1.e-8

C 2) Calculation of inverse Obukhov length scale
************************************************

          ol=obukhov(ps(ix,jy,1,n),tt2(ix,jy,1,n),td2(ix,jy,1,n),
     +    tth(ix,jy,2),ustar(ix,jy,1,n),sshf(ix,jy,1,n),akm,bkm)
          if (ol.ne.0.) then
            oli(ix,jy,1,n)=1./ol
          else
            oli(ix,jy,1,n)=99999.
          endif


C 3) Calculation of convective velocity scale and mixing height
***************************************************************

          do 20 i=1,nuvz
            ulev(i)=uuh(ix,jy,i)
            vlev(i)=vvh(ix,jy,i)
            ttlev(i)=tth(ix,jy,i)
20          qvlev(i)=qvh(ix,jy,i)

          call richardson(ps(ix,jy,1,n),ustar(ix,jy,1,n),ttlev,qvlev,
     +    ulev,vlev,nuvz,akz,bkz,sshf(ix,jy,1,n),tt2(ix,jy,1,n),
     +    td2(ix,jy,1,n),hmix1,wstar(ix,jy,1,n),hmixplus,
     +    zbotcv(ix,jy,1,n),ztopcv(ix,jy,1,n),lconvection)


          if(lsubgrid.eq.1) then
            subsceff=min(excessoro(ix,jy),hmixplus)
          else
            subsceff=0
          endif
*
* CALCULATE HMIX EXCESS ACCORDING TO SUBGRIDSCALE VARIABILITY AND STABILITY
*
          hmix(ix,jy,1,n)=hmix(ix,jy,1,n)+subsceff
          hmix(ix,jy,1,n)=max(hmixmin,hmix(ix,jy,1,n)) ! minim PBL height 150 m
          hmix(ix,jy,1,n)=min(hmixmax,hmix(ix,jy,1,n)) ! maxim PBL height 3500 m

C 4) Calculation of dry deposition velocities
*********************************************

          if (DRYDEP) then
            z0(4)=0.016*ustar(ix,jy,1,n)*ustar(ix,jy,1,n)/ga
            z0(9)=0.016*ustar(ix,jy,1,n)*ustar(ix,jy,1,n)/ga

C Calculate relative humidity at surface
****************************************
            rh=ew(td2(ix,jy,1,n))/ew(tt2(ix,jy,1,n))

            call getvdep(n,ix,jy,ustar(ix,jy,1,n),
     +      tt2(ix,jy,1,n),ps(ix,jy,1,n),1./oli(ix,jy,1,n),
     +      ssr(ix,jy,1,n),rh,lsprec(ix,jy,1,n)+convprec(ix,jy,1,n),vd)

            do 30 i=1,nspec
30            vdep(ix,jy,i,n)=vd(i)

          endif

10        continue


      return
      end
