      subroutine gridcheck(oronew,error)
***********************************************************************
*                                                                     * 
*             TRAJECTORY MODEL SUBROUTINE GRIDCHECK                   *
*             FLEXPART VERSION -> DO NOT USE IN FLEXTRA, PRAEPRO      *
*                                                                     *
***********************************************************************
*                                                                     * 
*             AUTHOR:      G. WOTAWA                                  *
*             DATE:        1997-08-06                                 *
*             LAST UPDATE: 1997-10-10                                 *
*                                                                     * 
*             Update:      1999-02-08, global fields allowed, A. Stohl* 
*                                                                     * 
***********************************************************************
*                                                                     *
* DESCRIPTION:                                                        *
*                                                                     *
* THIS SUBROUTINE DETERMINES THE GRID SPECIFICATIONS (LOWER LEFT      *
* LONGITUDE, LOWER LEFT LATITUDE, NUMBER OF GRID POINTS, GRID DIST-   *
* ANCE AND VERTICAL DISCRETIZATION OF THE ECMWF MODEL) FROM THE       *
* GRIB HEADER OF THE FIRST INPUT FILE. THE CONSISTANCY (NO CHANGES    *
* WITHIN ONE FLEXPART RUN) IS CHECKED IN THE ROUTINE "READWIND" AT    *
* ANY CALL.                                                           *
*                                                                     *
* OUTPUT       error .true.   - can not read grid specifications      *
*              error .false.  - normal                                *
*              oronew .true.  - Terrain heights given in grib files   *
*              oronew .false. - Terrain heights not specified in the  *
*                               grib files (old file standard)        *
*                                                                     * 
* XLON0                geographical longitude of lower left gridpoint *
* YLAT0                geographical latitude of lower left gridpoint  *
* NX                   number of grid points x-direction              *
* NY                   number of grid points y-direction              *
* DX                   grid distance x-direction                      *
* DY                   grid distance y-direction                      *
* NUVZ                 number of grid points for horizontal wind      *
*                      components in z direction                      *
* NWZ                  number of grid points for vertical wind        *
*                      component in z direction                       *
* sizesouth, sizenorth give the map scale (i.e. number of virtual grid*
*                      points of the polar stereographic grid):       *
*                      used to check the CFL criterion                *
* UVHEIGHT(1)-         heights of gridpoints where u and v are        *
* UVHEIGHT(NUVZ)       given                                          *
* WHEIGHT(1)-          heights of gridpoints where w is given         *
* WHEIGHT(NWZ)                                                        *
*                                                                     *
***********************************************************************
*
      include 'includepar'
      include 'includecom'

      integer i,ifn,ifield,j,k,iumax,iwmax,numskip
      real xaux1,xaux2,yaux1,yaux2,sizesouth,sizenorth,xauxa
      real plev(nwzmax),help
      logical error,oronew

* VARIABLES AND ARRAYS NEEDED FOR GRIB DECODING

C dimension of isec2 at least (22+n), where n is the number of parallels or
C meridians in a quasi-regular (reduced) Gaussian or lat/long grid

C dimension of zsec2 at least (10+nn), where nn is the number of vertical
C coordinate parameters

      integer isec0(2),isec1(56),isec2(22+nxmax+nymax),isec3(2)
      integer isec4(64),inbuff(jpack),ilen,iswap,ierr,iword,lunit
      real zsec2(60+2*nuvzmax),zsec3(2),zsec4(jpunp)
      character*1 yoper,opt
      data yoper/'D'/


      error=.false.
      oronew=.false.
      iumax=0
      iwmax=0
*
      if(ideltas.gt.0) then
        ifn=1
      else
        ifn=numbwf
      endif
*
* OPENING OF DATA FILE (GRIB CODE)
*
5     call pbopen(lunit,path(3)(1:len(3))//wfname(ifn),'r',ierr)
      if(ierr.lt.0) goto 999

      ifield=0   
10    ifield=ifield+1
*
* GET NEXT FIELDS
*
      call pbgrib(lunit,inbuff,jpack,ilen,ierr)
      if(ierr.eq.-1) goto 30    ! EOF DETECTED
      if(ierr.lt.-1) goto 999   ! ERROR DETECTED
 
      ierr=1

C Check whether we are on a little endian or on a big endian computer
*********************************************************************

      if (inbuff(1).eq.1112101447) then         ! little endian, swap bytes
        iswap=1+ilen/4
        call swap32(inbuff,iswap)
      else if (inbuff(1).ne.1196575042) then    ! big endian
        stop 'subroutine gridcheck: corrupt GRIB data'
      endif

      call gribex(isec0,isec1,isec2,zsec2,isec3,zsec3,isec4,
     &            zsec4,jpunp,inbuff,jpack,iword,yoper,ierr)
      if (ierr.ne.0) goto 10    ! ERROR DETECTED

      if(ifield.eq.1) then
        nxfield=isec2(2)
        ny=isec2(3)
        xaux1=float(isec2(5))/1000.
        xaux2=float(isec2(8))/1000.
        if((xaux1.eq.0.).and.(xaux2.eq.-1.0)) then ! NCEP DATA FROM 0 TO
          xaux1=-179.0                             ! 359 DEG EAST ->
          xaux2=180.0                              ! TRANSFORMED TO -179
        endif                                      ! TO 180 DEG EAST
        if (xaux1.gt.180) xaux1=xaux1-360.0        ! (in subroutine readwind)
        if (xaux2.gt.180) xaux2=xaux2-360.0
        if (xaux1.lt.-180) xaux1=xaux1+360.0
        if (xaux2.lt.-180) xaux2=xaux2+360.0
        if (xaux2.lt.xaux1) xaux2=xaux2+360.
        yaux1=float(isec2(7))/1000.
        yaux2=float(isec2(4))/1000.
        xlon0=xaux1
        ylat0=yaux1
        dx=(xaux2-xaux1)/float(nxfield-1)
        dy=(yaux2-yaux1)/float(ny-1)
        dxconst=180./(dx*r_earth*pi)
        dyconst=180./(dy*r_earth*pi)


C Check whether fields are global
C If they contain the poles, specify polar stereographic map
C projections using the stlmbr- and stcm2p-calls
************************************************************

        xauxa=abs(xaux2+dx-360.-xaux1)
        if (xauxa.lt.0.001) then
          nx=nxfield+1                 ! field is cyclic
          xglobal=.true.
        else
          nx=nxfield
          xglobal=.false.
        endif
        xauxa=abs(yaux1+90.)
        if (xglobal.and.xauxa.lt.0.001) then
          sglobal=.true.               ! field contains south pole
C Enhance the map scale by factor 3 (*2=6) compared to north-south
C map scale
          sizesouth=6.*(switchsouth+90.)/dy
          call stlmbr(southpolemap,-90.,0.)
          call stcm2p(southpolemap,0.,0.,switchsouth,0.,sizesouth,
     +    sizesouth,switchsouth,180.)
          switchsouthg=(switchsouth-ylat0)/dy
        else
          sglobal=.false.
          switchsouthg=999999.
        endif
        xauxa=abs(yaux2-90.)
        if (xglobal.and.xauxa.lt.0.001) then
          nglobal=.true.               ! field contains north pole
C Enhance the map scale by factor 3 (*2=6) compared to north-south
C map scale
          sizenorth=6.*(90.-switchnorth)/dy
          call stlmbr(northpolemap,90.,0.)
          call stcm2p(northpolemap,0.,0.,switchnorth,0.,sizenorth,
     +    sizenorth,switchnorth,180.)
          switchnorthg=(switchnorth-ylat0)/dy
        else
          nglobal=.false.
          switchnorthg=999999.
        endif
      endif

      if((isec1(6).eq.007).and.(isec1(7).eq.001)) oronew=.true.
      if((isec1(6).eq.33).and.(isec1(7).eq.100)) then
        iumax=iumax+1
        plev(iumax)=float(isec1(8))*100.0
      endif
     
      if((isec1(6).eq.007).and.(isec1(7).eq.001)) then
        do 20 j=0,ny-1
          do 21 i=0,nxfield-1
            oro(i,j)=0.0        ! ISOBARIC SURFACES: TERRAIN DISREGARDED
21          excessoro(i,j)=0.0  ! ISOBARIC SURFACES: SUBGRID TERRAIN DISREGARDED
          if (xglobal) oro(nx-1,j)=oro(0,j)
          if (xglobal) excessoro(nx-1,j)=excessoro(0,j)
20        continue
      endif

      if((isec1(6).eq.081).and.(isec1(7).eq.001)) then
* LAND SEA MASK
        do 25 j=0,ny-1
          do 26 i=0,nxfield-1
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              lsm(179+i,j)=help
            else
              lsm(i-181,j)=help
            endif
26        continue 
          if (xglobal) lsm(nx-1,j)=lsm(0,j)
25        continue
      endif

      goto 10                      !! READ NEXT LEVEL OR PARAMETER
*
* CLOSING OF INPUT DATA FILE
*
30    call pbclose(lunit,ierr)     !! FINISHED READING / CLOSING GRIB FILE

      nuvz=iumax
      nwz =iumax
      nlev_ec=iumax

      if (nx.gt.nxmax) then                         
      write(*,*) 'Trajectory  model error: Too many grid points'//
     &           ' in x direction.'
      write(*,*) 'Reduce resolution of wind fields.'
      error=.true.
      return
      endif

      if (ny.gt.nymax) then                         
      write(*,*) 'FLEXPART error: Too many grid points in y direction.'
      write(*,*) 'Reduce resolution of wind fields.'
      error=.true.
      return
      endif

      if (nuvz.gt.nuvzmax) then                         
      write(*,*) 'FLEXPART error: Too many u,v grid points in z '//
     +'direction.'
      write(*,*) 'Reduce resolution of wind fields.'
      error=.true.
      return
      endif

      if (nwz.gt.nwzmax) then                         
      write(*,*) 'FLEXPART error: Too many w grid points in z '//
     +'direction.'
      write(*,*) 'Reduce resolution of wind fields.'
      error=.true.
      return
      endif

C Output of grid info
*********************

      write(*,*)
      write(*,*)
      write(*,'(a,2i7)') '# of vertical levels in ECMWF data: ',
     +nuvz,nwz
      write(*,*)
      write(*,'(a)') 'Mother domain:'
      write(*,'(a,f10.1,a1,f10.1,a,f10.1)') '  Longitude range: ',
     +xlon0,' to ',xlon0+(nx-1)*dx,'   Grid distance: ',dx
      write(*,'(a,f10.1,a1,f10.1,a,f10.1)') '  Latitude range: ',
     +ylat0,' to ',ylat0+(ny-1)*dy,'   Grid distance: ',dy
      write(*,*)


* CALCULATE VERTICAL DISCRETIZATION OF ECMWF MODEL
* PARAMETER akm,bkm DESCRIBE THE HYBRID "ETA" COORDINATE SYSTEM

      numskip=nlev_ec-nuvz  ! number of ecmwf model layers not used
                            ! by trajectory model
      do 40 i=1,nwz
        j=numskip+i
        k=nlev_ec+1+numskip+i
        akm(nwz-i+1)=plev(nwz-i+1)
40      bkm(nwz-i+1)=0.0

*
* CALCULATION OF AKZ, BKZ
* AKZ,BKZ: model discretization parameters at the center of each model
*          layer
*
C Assign the 10 m winds to an artificial model level with akz=0 and bkz=1.0,
C i.e. ground level
******************************************************************************
 
      do 90 i=1,nuvz
         akz(i)=akm(i)
         bkz(i)=bkm(i)
90    continue

* Calculate aknew,bknew - model discretization parameters for increased
* vertical resolution
*
      nz=nuvz
      if (nz.gt.nzmax) stop 'nzmax too small'
      do 100 i=1,nz
        aknew(i)=akm(i)
100     bknew(i)=bkm(i)

      return

999   write(*,*)  
      write(*,*) ' ###########################################'//
     &           '###### '
      write(*,*) '       TRAJECTORY MODEL SUBROUTINE GRIDCHECK:'
      write(*,*) ' CAN NOT OPEN INPUT DATA FILE '//wfname(ifn)
      write(*,*) ' ###########################################'//
     &           '###### '
      write(*,*)
      write(*,'(a)') '!!! PLEASE INSERT A NEW CD-ROM AND   !!!'
      write(*,'(a)') '!!! PRESS ANY KEY TO CONTINUE...     !!!'
      write(*,'(a)') '!!! ...OR TERMINATE FLEXTRA PRESSING !!!'
      write(*,'(a)') '!!! THE "X" KEY...                   !!!'
      write(*,*)
      read(*,'(a)') opt
      if(opt.eq.'X') then
        error=.true.
      else
        goto 5
      endif

      return
      end
