      subroutine readoro(oronew,error)
C                          i      o
********************************************************************************
*                                                                              *
*     This routine reads the orography used by the ECMWF model.                *
*                                                                              *
*  This subroutine only works for the mother domain in order to be able to     *
*  use "old" data, where standard deviation of orography was not available.    *
*                                                                              *
*     Authors: A. Stohl                                                        *
*                                                                              *
*     30 April 1994                                                            *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
* error                .true., if file OROGRAPHY cannot be opened              *
* nx,ny                number of grid points in x and y direction, respectively*
* oro(0:nxmax-1,0:nymax-1) [m]  orography of ECMWF model                       *
* lsm(0:nxmax-1,0:nymax-1)    land sea mask (0 over sea, 1 over land)          *
*                                                                              *
* Constants:                                                                   *
* unitoro              unit connected to file OROGRAPHY                        *
* unitlsm              unit connected to file LANDSEAMASK                      *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'

      logical oronew,error
      integer ix,jy


      error=.false.

C Open the orography file and read grid
***************************************

      if (.not.oronew) then
        open(unitoro,file=path(3)(1:len(3))//'OROGRAPHY',
     +  status='old',err=999)
        do 10 ix=0,nx-1
          do 10 jy=0,ny-1
10          read(unitoro,*) oro(ix,jy)
        close(unitoro)

C Open the land sea mask file and read grid
*******************************************

        open(unitlsm,file=path(3)(1:len(3))//'LANDSEAMASK',
     +  status='old',err=998)
        do 20 ix=0,nx-1
          do 20 jy=0,ny-1
20          read(unitoro,*) lsm(ix,jy)
        close(unitlsm)
      endif


      return

998   write(*,*) ' #### FLEXPART MODEL ERROR! FILE "LANDSEAMASK"  #### '
      write(*,*) ' #### CANNOT BE OPENED IN THE DIRECTORY       #### '
      write(*,*) ' #### $FLEXPART/windfields                    #### '
      error=.true.
      return

999   write(*,*) ' #### FLEXPART MODEL ERROR! FILE "OROGRAPHY"  #### ' 
      write(*,*) ' #### CANNOT BE OPENED IN THE DIRECTORY       #### '
      write(*,*) ' #### $FLEXPART/windfields                    #### '
      error=.true.
      return

      end
