      subroutine readwind(indj,n,uuh,vvh,tth,qvh,wwh)
***********************************************************************
*                                                                     * 
*             TRAJECTORY MODEL SUBROUTINE READWIND                    *
*                                                                     *
***********************************************************************
*                                                                     * 
*             AUTHOR:      G. WOTAWA                                  *
*             DATE:        1997-08-05                                 *
*             LAST UPDATE: 2000-10-17, Andreas Stohl                  *
*                                                                     * 
***********************************************************************
*                                                                     *
* DESCRIPTION:                                                        *
*                                                                     *
* READING OF ECMWF METEOROLOGICAL FIELDS FROM INPUT DATA FILES. THE   *
* INPUT DATA FILES ARE EXPECTED TO BE AVAILABLE IN GRIB CODE          *
*                                                                     *
* INPUT:                                                              *
* indj               indicates number of the wind field to be read in *
* n                  temporal index for meteorological fields (1 to 3)*
*                                                                     *
* IMPORTANT VARIABLES FROM COMMON BLOCK:                              *
*                                                                     *
* wfname             File name of data to be read in                  *
* nx,ny,nuvz,nwz     expected field dimensions                        *
* nlev_ec            number of vertical levels ecmwf model            *
* uu,vv,ww           wind fields                                      *
* tt,qv              temperature and specific humidity                *
* ps                 surface pressure                                 *
*                                                                     *
***********************************************************************
*

      include 'includepar'
      include 'includecom'

      real uuh(0:nxmax-1,0:nymax-1,nuvzmax)
      real vvh(0:nxmax-1,0:nymax-1,nuvzmax)
      real tth(0:nxmax-1,0:nymax-1,nuvzmax)
      real qvh(0:nxmax-1,0:nymax-1,nuvzmax)
      real wwh(0:nxmax-1,0:nymax-1,nwzmax)
      integer numpt,numpu,numpv,numpw,numprh
      integer indj,i,j,k,n,ifield,iumax,iwmax,lunit,induvz
      integer indwz

* VARIABLES AND ARRAYS NEEDED FOR GRIB DECODING

C dimension of isec2 at least (22+n), where n is the number of parallels or
C meridians in a quasi-regular (reduced) Gaussian or lat/long grid

C dimension of zsec2 at least (10+nn), where nn is the number of vertical
C coordinate parameters

      integer isec0(2),isec1(56),isec2(22+nxmax+nymax),isec3(2)
      integer isec4(64),inbuff(jpack),ilen,iswap,ierr,iword
      real zsec2(60+2*nuvzmax),zsec3(2),zsec4(jpunp)
      real xaux,yaux,xaux0,yaux0
      real ewss(0:nxmax-1,0:nymax-1),nsss(0:nxmax-1,0:nymax-1)
      real hlev1,ff10m,fflev1,help,temp,plev,ew,elev
      real ulev1(0:nxmax-1,0:nymax-1),vlev1(0:nxmax-1,0:nymax-1)
      real tlev1(0:nxmax-1,0:nymax-1)

      character*1 yoper,opt
      logical error,hflswitch,strswitch

      data yoper/'D'/

      hflswitch=.false.
      strswitch=.false.
      iumax=0
      iwmax=0

*
* OPENING OF DATA FILE (GRIB CODE)
*
5     call pbopen(lunit,path(3)(1:len(3))//wfname(indj),'r',ierr)
      if(ierr.lt.0) goto 999

      numpt=0
      numpu=0
      numpv=0
      numpw=0
      numprh=0
      ifield=0
10    ifield=ifield+1
*
* GET NEXT FIELDS
*
      call pbgrib(lunit,inbuff,jpack,ilen,ierr)
      if(ierr.eq.-1) goto 50    ! EOF DETECTED
      if(ierr.lt.-1) goto 888   ! ERROR DETECTED

      ierr=1

C Check whether we are on a little endian or on a big endian computer
*********************************************************************

      if (inbuff(1).eq.1112101447) then         ! little endian, swap bytes
        iswap=1+ilen/4
        call swap32(inbuff,iswap)
      else if (inbuff(1).ne.1196575042) then    ! big endian
        stop 'subroutine gridcheck: corrupt GRIB data'
      endif


      call gribex(isec0,isec1,isec2,zsec2,isec3,zsec3,isec4,
     &            zsec4,jpunp,inbuff,jpack,iword,yoper,ierr)
      if (ierr.ne.0) goto 10    ! ERROR DETECTED

      if(ifield.eq.1) then

* CHECK GRID SPECIFICATIONS

        if(isec2(2).ne.nxfield) stop 'READWIND: NX NOT CONSISTENT'
        if(isec2(3).ne.ny) stop 'READWIND: NY NOT CONSISTENT'
        xaux=float(isec2(5))/1000.
        if(xaux.eq.0.) xaux=-179.0     ! NCEP DATA
        yaux=float(isec2(7))/1000.
        xaux0=xlon0
        yaux0=ylat0
        if(xaux.lt.0.) xaux=xaux+360.
        if(yaux.lt.0.) yaux=yaux+360.
        if(xaux0.lt.0.) xaux0=xaux0+360.
        if(yaux0.lt.0.) yaux0=yaux0+360.
        if(xaux.ne.xaux0)
     &    stop 'READWIND: LOWER LEFT LONGITUDE NOT CONSISTENT'
        if(yaux.ne.yaux0)
     &    stop 'READWIND: LOWER LEFT LATITUDE NOT CONSISTENT'
      endif

      do 20 j=0,ny-1
        do 20 i=0,nxfield-1

          if((isec1(6).eq.011).and.(isec1(7).eq.100)) then
* TEMPERATURE
            if((i.eq.0).and.(j.eq.0)) numpt=numpt+1
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              tth(179+i,j,numpt)=help
            else
              tth(i-181,j,numpt)=help
            endif
          endif
          if((isec1(6).eq.033).and.(isec1(7).eq.100)) then
* U VELOCITY
            if((i.eq.0).and.(j.eq.0)) numpu=numpu+1
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              uuh(179+i,j,numpu)=help
            else
              uuh(i-181,j,numpu)=help
            endif
          endif
          if((isec1(6).eq.034).and.(isec1(7).eq.100)) then
* V VELOCITY
            if((i.eq.0).and.(j.eq.0)) numpv=numpv+1
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              vvh(179+i,j,numpv)=help
            else
              vvh(i-181,j,numpv)=help
            endif
          endif
          if((isec1(6).eq.039).and.(isec1(7).eq.100)) then
* W VELOCITY
            if((i.eq.0).and.(j.eq.0)) numpw=numpw+1
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              wwh(179+i,j,numpw)=help
            else
              wwh(i-181,j,numpw)=help
            endif
          endif
          if((isec1(6).eq.052).and.(isec1(7).eq.100)) then
* RELATIVE HUMIDITY -> CONVERT TO SPECIFIC HUMIDITY
            if((i.eq.0).and.(j.eq.0)) numprh=numprh+1
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              qvh(179+i,j,numprh)=help
            else
              qvh(i-181,j,numprh)=help
            endif
          endif
          if((isec1(6).eq.001).and.(isec1(7).eq.001)) then
* SURFACE PRESSURE
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              ps(179+i,j,1,n)=help
            else
              ps(i-181,j,1,n)=help
            endif
          endif
          if((isec1(6).eq.007).and.(isec1(7).eq.001)) then
            oro(i,j)=0.0       ! ISOBARIC SURFACES: TERRAIN DISREGARDED
            excessoro(i,j)=0.0 ! ISOBARIC SURFACES: SUBGRID TERRAIN DISREGARDED
          endif
          if((isec1(6).eq.081).and.(isec1(7).eq.001)) then
* LAND SEA MASK
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              lsm(179+i,j)=help
            else
              lsm(i-181,j)=help
            endif
          endif
          if((isec1(6).eq.002).and.(isec1(7).eq.102)) then
* MSL PRESSURE
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              msl(179+i,j,1,n)=help
            else
              msl(i-181,j,1,n)=help
            endif
          endif
          if((isec1(6).eq.071).and.(isec1(7).eq.244)) then
* TOTAL CLOUD COVER
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              tcc(179+i,j,1,n)=help
            else
              tcc(i-181,j,1,n)=help
            endif
          endif
          if((isec1(6).eq.033).and.(isec1(7).eq.105).and.
     &      (isec1(8).eq.10)) then
* 10 M U VELOCITY
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              u10(179+i,j,1,n)=help
            else
              u10(i-181,j,1,n)=help
            endif
          endif
          if((isec1(6).eq.033).and.(isec1(7).eq.107)) then
* U VELOCITY LOWEST SIGMA LEVEL
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              ulev1(179+i,j)=help
            else
              ulev1(i-181,j)=help
            endif
          endif
          if((isec1(6).eq.034).and.(isec1(7).eq.105).and.
     &      (isec1(8).eq.10)) then
* 10 M V VELOCITY
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              v10(179+i,j,1,n)=help
            else
              v10(i-181,j,1,n)=help
            endif
          endif
          if((isec1(6).eq.034).and.(isec1(7).eq.107)) then
* V VELOCITY LOWEST SIGMA LEVEL
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              vlev1(179+i,j)=help
            else
              vlev1(i-181,j)=help
            endif
          endif
          if((isec1(6).eq.011).and.(isec1(7).eq.105).and.
     &      (isec1(8).eq.02)) then
* 2 M TEMPERATURE
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              tt2(179+i,j,1,n)=help
            else
              tt2(i-181,j,1,n)=help
            endif
          endif
          if((isec1(6).eq.011).and.(isec1(7).eq.107)) then
* TEMPERATURE LOWEST SIGMA LEVEL
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              tlev1(179+i,j)=help
            else
              tlev1(i-181,j)=help
            endif
          endif

          if((isec1(6).eq.221).and.(isec1(7).eq.001)) then
* MIXING HEIGHT
            help=zsec4(nxfield*(ny-j-1)+i+1)
            if(i.le.180) then
              hmix(179+i,j,1,n)=help
            else
              hmix(i-181,j,1,n)=help
            endif
          endif

20    continue

      if((isec1(6).eq.33).and.(isec1(7).eq.100)) iumax=iumax+1

      goto 10                      !! READ NEXT LEVEL OR PARAMETER
*
* CLOSING OF INPUT DATA FILE
*
50    call pbclose(lunit,ierr)     !! FINNISHED READING / CLOSING GRIB FILE

* TRANSFORM RH TO SPECIFIC HUMIDITY AS NEEDED

      do 65 j=0,ny-1
        do 65 i=0,nxfield-1
          lsprec(i,j,1,n)=0.0   ! Analysis data contain no precip
          convprec(i,j,1,n)=0.0 ! Analysis data contain no precip
          ssr(i,j,1,n)=0.0      ! Analysis data contain no fluxes
          sd(i,j,1,n)=0.0       ! Snow depth disregarded
          td2(i,j,1,n)=tt2(i,j,1,n)    ! Dew point is set to temp.
          do 65 k=1,nuvz
            help=qvh(i,j,k)
            temp=tth(i,j,k)
            plev=akm(k)
            elev=ew(temp)*help/100.0
            qvh(i,j,k)=xmwml*(elev/(plev-((1.0-xmwml)*elev)))
65    continue


C For global fields, assign rightmost grid point the value of the
C leftmost point
*****************************************************************

      if (xglobal) then
        do 80 j=0,ny-1
          ewss(nx-1,j)=ewss(0,j)
          nsss(nx-1,j)=nsss(0,j)
          oro(nx-1,j)=oro(0,j)
          excessoro(nx-1,j)=excessoro(0,j)
          lsm(nx-1,j)=lsm(0,j)
          ps(nx-1,j,1,n)=ps(0,j,1,n)
          sd(nx-1,j,1,n)=sd(0,j,1,n)
          msl(nx-1,j,1,n)=msl(0,j,1,n)
          tcc(nx-1,j,1,n)=tcc(0,j,1,n)
          u10(nx-1,j,1,n)=u10(0,j,1,n)
          v10(nx-1,j,1,n)=v10(0,j,1,n)
          tt2(nx-1,j,1,n)=tt2(0,j,1,n)
          td2(nx-1,j,1,n)=td2(0,j,1,n)
          lsprec(nx-1,j,1,n)=lsprec(0,j,1,n)
          convprec(nx-1,j,1,n)=convprec(0,j,1,n)
          sshf(nx-1,j,1,n)=sshf(0,j,1,n)
          ssr(nx-1,j,1,n)=ssr(0,j,1,n)
          do 81 induvz=1,nuvz
            tth(nx-1,j,induvz)=tth(0,j,induvz)
            qvh(nx-1,j,induvz)=qvh(0,j,induvz)
            uuh(nx-1,j,induvz)=uuh(0,j,induvz)
81          vvh(nx-1,j,induvz)=vvh(0,j,induvz)
          do 80 indwz=1,nwz
80          wwh(nx-1,j,indwz)=wwh(0,j,indwz)
      endif

C CALCULATE USTAR AND SSHF USING THE PROFILE METHOD
***************************************************

        do 1500 i=0,nx-1
          do 1500 j=0,ny-1
            hlev1=30.0                     ! HEIGTH OF FIRST MODEL LAYER
            ff10m= sqrt(u10(i,j,1,n)**2+v10(i,j,1,n)**2)
            fflev1=sqrt(ulev1(i,j)**2+vlev1(i,j)**2)
            call pbl_profile(ps(i,j,1,n),td2(i,j,1,n),hlev1,
     &                       tt2(i,j,1,n),tlev1(i,j),ff10m,fflev1,
     &                       surfstr(i,j,1,n),sshf(i,j,1,n))
            if(sshf(i,j,1,n).gt.200.) sshf(i,j,1,n)=200.
            if(sshf(i,j,1,n).lt.-400.) sshf(i,j,1,n)=-400.
1500        continue


      if(iumax.ne.nuvz) stop 'READWIND: NUVZ NOT CONSISTENT'
      if(iumax.ne.nwz)    stop 'READWIND: NWZ NOT CONSISTENT'

      return    
888   write(*,*) ' #### FLEXPART MODEL ERROR! WINDFIELD         #### ' 
      write(*,*) ' #### ',wfname(indj),'                    #### '
      write(*,*) ' #### IS NOT GRIB FORMAT !!!                  #### '
      stop 'Execution terminated'
999   write(*,*) ' #### FLEXPART MODEL ERROR! WINDFIELD         #### ' 
      write(*,*) ' #### ',wfname(indj),'                    #### '
      write(*,*) ' #### CANNOT BE OPENED !!!                    #### '
      write(*,*)
      write(*,'(a)') '!!! PLEASE INSERT A NEW CD-ROM AND   !!!'
      write(*,'(a)') '!!! PRESS ANY KEY TO CONTINUE...     !!!'
      write(*,'(a)') '!!! ...OR TERMINATE FLEXTRA PRESSING !!!'
      write(*,'(a)') '!!! THE "X" KEY...                   !!!'
      write(*,'(a)') '!!! PLEASE CHECK CD-ROM LABEL AND    !!!'
      write(*,'(a)') '!!! CORRECT FILE "PATHNAMES"...      !!!'
      write(*,*)
      read(*,'(a)') opt
      if(opt.eq.'X') then
        stop 'Execution terminated'
      else
        call readpaths(error)
        if(error)
     &    stop 'Error reading "pathnames" --> execution terminated'
        goto 5
      endif

      end
